@@ -1,4 +1,4 @@
-#$Id: Prereq.pm,v 1.1 2002/09/12 10:11:07 comdog Exp $
+#$Id: Prereq.pm,v 1.2 2002/09/22 21:19:35 comdog Exp $
 package Test::Prereq;
 use strict;
 
@@ -32,10 +32,11 @@ use vars qw($VERSION @EXPORT);
 $VERSION = '0.05';
 @EXPORT = qw( prereq_ok );
 
+use Carp qw(carp);
 use ExtUtils::MakeMaker;
 use File::Find::Rule;
-use Module::Info;
 use Module::CoreList;
+use Module::Info;
 use Test::Builder;
 
 my $Test = Test::Builder->new;
@@ -69,7 +70,7 @@ sub ExtUtils::MakeMaker::WriteMakefile
 
 =over 4
 
-=item prereq_ok( [ VERSION, [ NAME ] ] )
+=item prereq_ok( [ VERSION, [ NAME [, SKIP_ARRAY] ] ] )
 
 If you don't specify a version, prereq_ok assumes you want to compare
 the list of prerequisite modules to version 5.6.1.
@@ -86,19 +87,33 @@ Valid version come from Module::CoreList (which uses $[):
         5.006
         5.006001
 
+prereq_ok attempts to remove modules found in blib and 
+libraries found in t from the reported prerequisites.
+
+The optional third argument is an array reference to a list of
+names that prereq_ok should ignore. You might want to use this
+if your tests do funny things with require.
+
 =cut
 
 my $default_version = '5.006001';
-my $version = '5.006001';
+my $version         = '5.006001';
 
 sub prereq_ok
 	{
 	   $version  = shift || '5.006001';
 	my $name     = shift || 'Prereq test';
+	my $skip     = shift || [];
 	
 	$version = $default_version unless 
 		exists $Module::CoreList::version{$version};
 	
+	unless( UNIVERSAL::isa( $skip, 'ARRAY' ) )
+		{
+		carp( 'Third parameter to prereq_ok must be an array reference!' );
+		return;
+		}
+		
 	my $prereqs = _get_prereqs();
 	unless( $prereqs )
 		{
@@ -125,7 +140,27 @@ sub prereq_ok
 		{
 		delete $loaded->{$module};
 		}
-		
+
+	# remove modules found in distribution	
+	my $distro = _get_dist_modules( 'blib/lib' );
+	foreach my $module ( @$distro )
+		{
+		delete $loaded->{$module};
+		}
+
+	# remove modules found in test directory	
+	$distro = _get_test_libraries();
+	foreach my $module ( @$distro )
+		{
+		delete $loaded->{$module};
+		}
+
+	# remove modules in the skip array	
+	foreach my $module ( @$skip )
+		{
+		delete $loaded->{$module};
+		}
+				
 	if( keys %$loaded ) # stuff left in %loaded, oops!
 		{
 		$Test->ok( 0, $name );
@@ -167,13 +202,51 @@ sub _get_loaded_modules
 	return { map { $_, 1 } @found };
 	}
 
+sub _get_test_libraries
+	{
+	my $dirsep = "/";
+
+	my @files = 
+		map {
+			my $x = $_;
+			$x =~ s/^.*$dirsep//;
+			$x =~ s|$dirsep|::|g;
+			$x;
+			}
+			File::Find::Rule->file()->name( '*.pl' )->in( 't' );
+
+	return \@files;
+	}
+		
+sub _get_dist_modules
+	{
+	return unless( defined $_[0] and -d $_[0] );
+	
+	my $dirsep = "/";
+	
+	my @files = 
+		map { 
+			my $x = $_;
+			$x =~ s/^$_[0]($dirsep)?//;
+			$x =~ s/\.pm$//;
+			$x =~ s|$dirsep|::|g;
+			$x;
+			}
+			File::Find::Rule->file()->name( '*.pm' )->in( $_[0] );
+			
+	#print STDERR "Found in blib @files\n";
+	
+	return \@files;
+	}
+	
 sub _get_from_file
 	{
 	my $file = shift;
 	
 	my $module = Module::Info->new_from_file( $file );
+	
 	my @used = $module->modules_used;
-		
+	
 	my @modules = 
 		sort 
 		grep { not exists $Module::CoreList::version{$version}{$_} }
